function sot_cat2files, catrecs, urls=urls, doy=doy, debug=debug
;
;+
;   Name: sot_cat2files
;
;   Purpose: map from sot catalog entry to files -or- urls
;
;   Input Parameters:
;      catrecs - desired FPP/SOT catalog entries
;
;   Output Parameters:
;      function returns implied nfs path -or- FPP data center urls
;
;   Keyword Paramters:
;      urls - if set, return URLS instead of NFS filenames
;      doy - if set, ...yyyyDdoy_hhmmss.msec instead of
;                    ...yyyymmdd_hhmmss.msec (pre launch test data?)
;   
;   History:
;      10-Apr-2006 - S.L.Freeland - called from sot_time2files( w/CATALOG) 
;      11-oct-2006 - S.L.Freeland - $FPP_DATA -> $SOT_DATA
;
;-
;

debug=keyword_set(debug)
sotdata=get_logenv('$SOT_DATA')
xrtdata=get_logenv('$XRT_DATA')
sbdata=get_logenv('$SOLARB_DATA')  ; common parent defined?


obsdir=strtrim(gt_tagval(catrecs,/obsdir,missing=''),2)

ssxrt=where(strupcase(obsdir) eq 'XRT',xcnt)

xrt=xcnt gt 0
case 1 of 
   keyword_set(urls): begin 
      top=get_logenv('SOLARB_LOCAL_SERVER') ; non LMSAL centers may define this 
      if top(0) eq '' then top='http://darts.isas.jaxa.jp/pub/solar/solarb/'
   endcase
   else: begin 
      case 1 of 
         file_exist(xrtdata) and xrt: top=xrtdata
         file_exist(sotdata): top=sotdata
         file_exist(sbdata): top=concat_dir(sbdata,'sot')
         else: top='/net/kokuten/archive/sci/' ; lmsal PP center default?
      endcase
      if not file_exist(top) then begin
         box_message,"No local SOT data??; use /URLS to return remote names(urls)"
         return,''
      endif
   endcase
endcase

ftd=anytim(catrecs.anytim_dobs,/ecs)
ft=strcompress(str_replace(str_replace(str_replace(ftd,' ','_'),'/',' '),':',' '),/remove)
if keyword_set(doy) then begin 
    doys=anytim2doy(anytim(catrecs.anytim_dobs,/int))
    sdoys=fstring(doys,format='("D",I3.3)')   
    ft=strmid(ft,0,4)+sdoys+strmid(ft,8,18)
endif

hours=gt_tagval(catrecs,/hour,missing='',found=found) 
if not found then hours=strmid(ftd,11,2)

yymmdd=strmid(ftd,0,10)
prefix=obsdir
ss=where(prefix eq 'FGFOCUS',sscnt)
if sscnt gt 0 then prefix(ss)='FG' 
fprefix=gt_tagval(catrecs,/FPREFIX,missing='')
ssdiff=where(fprefix ne '',sscnt)
if sscnt gt 0 then prefix(ssdiff)=fprefix(ssdiff)
if xcnt gt 0 then obsdir(ssxrt)=''
fname=yymmdd+'/'+obsdir+'/'+strtrim(catrecs.hour,2)+'/'+prefix+ft+'.fits'

subdir=(['sot/','xrt/'])(obsdir eq '') + '/level0/'
if strpos(top,'http') ne -1 then begin  
   fname=subdir+temporary(fname)
endif
  

retval=concat_dir(top,fname)


if debug then stop,'retval'
return,retval
end
